# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This file is part of the LibreOffice project.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

from uitest.framework import UITestCase
import time
from uitest.uihelper.common import get_state_as_dict, type_text
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.debug import sleep

#test Find Bar
class FindBar(UITestCase):

    def test_find_bar(self):

        self.ui_test.create_doc_in_start_center("writer")
        xWriterDoc = self.xUITest.getTopFocusWindow()
        xWriterEdit = xWriterDoc.getChild("writer_edit")

        # Type some lines to search for words on them
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"TEXT": "LibreOffice"}))
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"KEYCODE": "RETURN"}))
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"TEXT": "LibreOffice Writer"}))
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"KEYCODE": "RETURN"}))
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"TEXT": "LibreOffice Calc"}))
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"KEYCODE": "RETURN"}))
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"TEXT": "The Document Foundation"}))

        # open the Find Bar
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"KEYCODE": "CTRL+f"}))

        # Type the Word that we want to search for it
        xfind = xWriterDoc.getChild("find")
        xfind.executeAction("TYPE", mkPropertyValues({"TEXT": "Libre"}))

        # Select the Find Bar
        xfind_bar = xWriterDoc.getChild("FindBar")
        self.assertEqual(get_state_as_dict(xfind_bar)["ItemCount"], "14")

        # Press on FindAll in the Find Bar
        xfind_bar.executeAction("CLICK", mkPropertyValues({"POS": "5"}))  # 5 is FindAll id
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemID"], "5")
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemText"], "Find All")
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemCommand"], ".uno:FindAll")
        self.assertEqual(get_state_as_dict(xWriterEdit)["SelectedText"], "LibreLibreLibre")

        # Press on Find Next in the Find Bar
        xfind_bar.executeAction("CLICK", mkPropertyValues({"POS": "4"}))  # 4 is Find Next id
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemID"], "4")
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemText"], "Find Next")
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemCommand"], ".uno:DownSearch")
        self.assertEqual(get_state_as_dict(xWriterEdit)["SelectedText"], "Libre")

        # Press on Find Previous in the Find Bar
        xfind_bar.executeAction("CLICK", mkPropertyValues({"POS": "3"}))  # 3 is Find Previous id
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemID"], "3")
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemText"], "Find Previous")
        self.assertEqual(get_state_as_dict(xfind_bar)["CurrSelectedItemCommand"], ".uno:UpSearch")
        self.assertEqual(get_state_as_dict(xWriterEdit)["SelectedText"], "Libre")

        # Close the Find Bar
        xfind_bar.executeAction("CLICK", mkPropertyValues({"POS": "1"}))  # 1 is for close

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from uitest.framework import UITestCase
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict, type_text
import time
from uitest.debug import sleep
import org.libreoffice.unotest
import pathlib

#Bug 81292 - TABLE: Crashes on sorting table

def get_url_for_data_file(file_name):
   return pathlib.Path(org.libreoffice.unotest.makeCopyFromTDOC(file_name)).as_uri()

class tdf81292(UITestCase):

   def test_tdf81292_table_sort(self):
        writer_doc = self.ui_test.load_file(get_url_for_data_file("tdf81292.odt"))
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()
        #select whole table
        self.xUITest.executeCommand(".uno:SelectTable")
        #Tools - Sort
        self.ui_test.execute_dialog_through_command(".uno:SortDialog")
        xDialog = self.xUITest.getTopFocusWindow()
        xDown = xDialog.getChild("down1")
        xDown.executeAction("CLICK", tuple())
        xOK = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOK)
        #verify
        tables = document.getTextTables()
        table = tables[0]
        tableText = table.getCellByName("B3")
        b3 = tableText.getString()
        tableTextc1 = table.getCellByName("C1").getString()
        self.assertEqual(b3, "https://www.hightail.com/")
        self.assertEqual(tableTextc1, "inlognaam")
        #undo
        self.xUITest.executeCommand(".uno:Undo")
        b3 = tableText.getString()
        self.assertEqual(b3, "www.comicstripshop.com")

        self.ui_test.close_doc()
# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from uitest.framework import UITestCase
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict, type_text

import time, re
from uitest.debug import sleep
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import select_pos

class LibreLogoCompileTest(UITestCase):
   LIBRELOGO_PATH = "vnd.sun.star.script:LibreLogo|LibreLogo.py$%s?language=Python&location=share"
   LS = "#_@L_i_N_e@_#" # LibreLogo line separator for debug feature "jump to the bad LibreLogo program line"

   def createMasterScriptProviderFactory(self):
       xServiceManager = self.xContext.ServiceManager
       return xServiceManager.createInstanceWithContext(
           "com.sun.star.script.provider.MasterScriptProviderFactory",
           self.xContext)

   def getScript(self, command):
       xMasterScriptProviderFactory = self.createMasterScriptProviderFactory()
       document = self.ui_test.get_component()
       xScriptProvider = xMasterScriptProviderFactory.createScriptProvider(document)
       xScript = xScriptProvider.getScript(self.LIBRELOGO_PATH %command)
       self.assertIsNotNone(xScript, "xScript was not loaded")
       return xScript

   def test_compile_librelogo(self):
        # XScript interface to LibreLogo Python compiler
        xCompile = self.getScript("__compil__")

        for test in (
                # BASE COMMANDS
                ("FORWARD 1 forward 1 fd 1", "forward(1)\nforward(1)\nforward(1)"),
                ("BACK 1 back 1 bk 1", "backward(1)\nbackward(1)\nbackward(1)"),
                ("RIGHT 1 right 1 rt 1", "turnright(1)\nturnright(1)\nturnright(1)"),
                ("LEFT 1 left 1 lt 1", "turnleft(1)\nturnleft(1)\nturnleft(1)"),
                # COMMENTS
                ("fd 1; comment\n; full-line comment", "forward(1)\n" + self.LS),
                # MULTI-LINE COMMAND
                ("LABEL ~\n10 + 10", "label(10 + 10)\n" + self.LS),
                # UNIT SPECIFIERS
                # check specifier "pt" (point)
                ("fd 1pt", "forward(1)"),
                # check specifier "pt" (point)
                ("fd 0.5pt", "forward(0.5)"),
                # check "in" (inch)
                ("fd 1in", "forward(72.0)"),
                # check "cm" (centimeter)
                ("fd 1cm", "forward(%s)" % (1/(2.54/72))),
                # check "mm" (millimeter)
                ("fd 10mm", "forward(%s)" % (1/(2.54/72))),
                # angle
                ("rt 90°", "turnright(90)"),
                # clock-position
                ("rt 3h", "turnright(90.0)"),
                # CONDITION
                ("if a = 1 [ ]", "if a == 1 :\n __checkhalt__()"),
                # with else
                ("if a == 1 [ ] [ ]", "if a == 1 :\n __checkhalt__()\nelse:\n __checkhalt__()"),
                # LOOPS
                ("repeat 10 [ ]", "for REPCOUNT in range(1, 1+int(10 )):\n __checkhalt__()"),
                # endless loop
                ("repeat [ ]", "REPCOUNT0 = 1\nwhile True:\n __checkhalt__()\n REPCOUNT = REPCOUNT0\n REPCOUNT0 += 1"),
                # loop in loop
                ("repeat 10 [ repeat REPCOUNT [ a=1 ] ]", "for REPCOUNT in range(1, 1+int(10 )):\n __checkhalt__()\n for REPCOUNT in range(1, 1+int(REPCOUNT )):\n  __checkhalt__()\n  a=1"),
                # while
                ("WHILE REPCOUNT < 10 [ ]", "REPCOUNT2 = 1\nwhile REPCOUNT2 < 10 :\n __checkhalt__()\n REPCOUNT = REPCOUNT2\n REPCOUNT2 += 1"),
                # for
                ("FOR i in [1, 2, 3] [ ]", "REPCOUNT4 = 1\nfor i in [1, 2, 3] :\n __checkhalt__()\n REPCOUNT = REPCOUNT4\n REPCOUNT4 += 1"),
                ("FOR i IN RANGE COUNT 'letters' [ ]", "REPCOUNT6 = 1\nfor i in range(len(u'letters'),) :\n __checkhalt__()\n REPCOUNT = REPCOUNT6\n REPCOUNT6 += 1"),
                # PROCEDURE
                ("TO x\nLABEL 2\nEND", "global x\ndef x():\n __checkhalt__()\n %s\n label(2)\n %s" % (((self.LS),)*2)),
                # FUNCTION
                ("TO x\nOUTPUT 3\nEND", "global x\ndef x():\n __checkhalt__()\n %s\n return 3\n %s" % (((self.LS),)*2)),
                # PROCEDURE WITH ARGUMENTS
                ("TO x y\nLABEL y\nEND\nx 25", "global x\ndef x(y):\n __checkhalt__()\n %s\n label(y)\n %s\n%s\nx(25)" % (((self.LS),)*3)),
                ("TO x :y :z\nLABEL :y + :z\nEND\nx 25 26", "global x\ndef x(_y, _z):\n __checkhalt__()\n %s\n label(_y + _z)\n %s\n%s\nx(25, 26)" % (((self.LS),)*3)),
                ("TO x :y :z\nLABEL :y + :z\nEND\nx 25 :w", "global x\ndef x(_y, _z):\n __checkhalt__()\n %s\n label(_y + _z)\n %s\n%s\nx(25, _w)" % (((self.LS),)*3)),
                # UNICODE VARIABLE NAMES
                ("Erdős=1", "Erd__u__0151s=1"),
                # STRING CONSTANTS
                ("LABEL \"label", "label(u'label')"),
                ("LABEL “label”", "label(u'label')"),
                ("LABEL 'label'", "label(u'label')"),
                ("LABEL ‘label’", "label(u'label')"),
                # check apostrophe and quote usage within strings
                ("LABEL “label\’s”", "label(u'label’s')"),
                ("LABEL ““It\’s quote\’s...\””", "label(u'“It’s quote’s...”')"),
                ("LABEL ““It\\'s quote\\'s...\””", "label(u'“It\\'s quote\\'s...”')"),
                # CONVERSION OF LOGO EXPRESSIONS
                ("a=SIN 100 + COS 100", "a=sin(100 + cos(100))"),
                ("a=SIN(101) + COS(101)", "a=sin(101) + cos(101)"),
                ("a=(SIN 102) + (COS 102)", "a=(sin(102)) + (cos(102))"),
                ("a=SIN 103 + COS 103 - SQRT 103", "a=sin(103 + cos(103 - sqrt(103)))"),
                ("a=(SIN 104 + COS 104) - SQRT 104", "a=(sin(104 + cos(104))) - sqrt(104)"),
                # SIN(x) is Python-like, SIN (x) is Logo-like syntax
                ("a=SIN(105) + COS (105) - SQRT 105", "a=sin(105) + cos((105) - sqrt(105))"),
                ("a=COUNT [1, 2, 3]", "a=len([1, 2, 3])"),
                ("PRINT COUNT [1, 2, 3]", "Print(len([1, 2, 3]))"),
                ("PRINT 'TEXT: ' + 'CHAR'[0] + ' TEXT2: ' + variable[-1]", "Print(u'TEXT: ' + u'CHAR'[0] + u' TEXT2: ' + variable[-1])"),
                ("PRINT 'TEXT: ' + 'CHAR'[0][n] + ' TEXT2: ' + varia[len k][i+1]", "Print(u'TEXT: ' + u'CHAR'[0][n] + u' TEXT2: ' + varia[len(k)][i+1])"),
                ("a=SQRT COUNT [1, 2, 3]", "a=sqrt(len([1, 2, 3]))"),
                ("a=RANGE 1", "a=range(1,)"),
                ("a=RANGE 1 10", "a=range(1, 10,)"),
                ("a=RANGE 1 10 5", "a=range(1, 10, 5)"),
                ("a=RANDOM 40 + 120", "a=Random(40 + 120)"),
                ("a=RANDOM(40) + 120", "a=Random(40) + 120"),
                ("a=RANDOM [1, 2, 3]", "a=Random([1, 2, 3])"),
                ("a=[sin 90 + cos 15, cos 100 * x, sqrt 25 * 25]", "a=[sin(90 + cos(15)), cos(100 * x), sqrt(25 * 25)]"),
                ("a=[sin 90 + cos 15, cos 100 * x, sqrt 25 * 25]", "a=[sin(90 + cos(15)), cos(100 * x), sqrt(25 * 25)]"),
                ("a=[sin(90) + cos 15, cos(100) * x, sqrt(25) * 25]", "a=[sin(90) + cos(15), cos(100) * x, sqrt(25) * 25]"),
                ("TO x y z\nOUTPUT 3\nEND", "global x\ndef x(y, z):\n __checkhalt__()\n %s\n return 3\n %s" % (((self.LS),)*2)),
                ("TO x\nOUTPUT 3\nEND", "global x\ndef x():\n __checkhalt__()\n %s\n return 3\n %s" % (((self.LS),)*2)),
                ("TO f x y z\nOUTPUT x+y+z\nEND\na = [-sin -len f [-cos 45, 6] -len [1, 2, 3] -sin -90", "global f\ndef f(x, y, z):\n __checkhalt__()\n %s\n return x+y+z\n %s\n%s\na = [-sin(-len(f([-cos(45), 6], -len([1, 2, 3]), -sin(-90))))" % (((self.LS),)*3)),
                ("TO f x y z\nOUTPUT x+y+z\nEND\na = [sin len f [cos 45, 6] [1, 2, 3] sin 90", "global f\ndef f(x, y, z):\n __checkhalt__()\n %s\n return x+y+z\n %s\n%s\na = [sin(len(f([cos(45), 6], [1, 2, 3], sin(90))))" % (((self.LS),)*3)),
                ("TO f x y z\nLABEL x+y+z\nEND\nf len [1, cos 2, [65]] sqrt len [1, 2, 3, 4] sin 90 * cos 270", "global f\ndef f(x, y, z):\n __checkhalt__()\n %s\n label(x+y+z)\n %s\n%s\nf(len([1, cos(2), [65]]), sqrt(len([1, 2, 3, 4])), sin(90 * cos(270)))" % (((self.LS),)*3)),
                ("TO f x y z\nLABEL x+y+z\nEND\nf len([1, cos 2, [65]]) sqrt(len [1, 2, 3, 4]) sin(90) * cos 270", "global f\ndef f(x, y, z):\n __checkhalt__()\n %s\n label(x+y+z)\n %s\n%s\nf(len([1, cos(2), [65]]), sqrt(len([1, 2, 3, 4])), sin(90) * cos(270))" % (((self.LS),)*3)),
                ("TO f x y z\nLABEL x+y+z\nEND\nf (len [1, cos 2, [65]]) (sqrt len [1, 2, 3, 4]) (sin 90) * (cos 270)", "global f\ndef f(x, y, z):\n __checkhalt__()\n %s\n label(x+y+z)\n %s\n%s\nf((len([1, cos(2), [65]])), (sqrt(len([1, 2, 3, 4]))), (sin(90)) * (cos(270)))" % (((self.LS),)*3)),
                # arbitrary order of function definitions and calls
                ("f 1 1 f 2 2\nTO f x y\nPRINT x + y\nEND", "global f\nf(1, 1)\nf(2, 2)\n%s\ndef f(x, y):\n __checkhalt__()\n %s\n Print(x + y)\n %s" % (((self.LS),)*3)),
                ):
            compiled = xCompile.invoke((test[0],), (), ())[0]
            self.assertEqual(test[1], re.sub(r'(\n| +\n)+', '\n', re.sub(r'\( ', '(', compiled)).strip())

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from uitest.uihelper.common import get_state_as_dict
from libreoffice.uno.propertyvalue import mkPropertyValues

class tdf130629(UITestCase):

    def test_tdf130629(self):

        self.ui_test.create_doc_in_start_center("writer")

        document = self.ui_test.get_component()
        self.assertEqual(0, document.DrawPage.getCount())

        # Insert shape while pressing CTRL
        self.xUITest.executeCommandWithParameters(".uno:BasicShapes.diamond", mkPropertyValues({"KeyModifier": 8192}))

        self.assertEqual(1, document.DrawPage.getCount())

        # Undo twice
        self.xUITest.executeCommand(".uno:Undo")
        self.xUITest.executeCommand(".uno:Undo")

        self.assertEqual(0, document.DrawPage.getCount())

        # Shape toolbar is active, use esc before inserting a new shape
        xWriterDoc = self.xUITest.getTopFocusWindow()
        xWriterEdit = xWriterDoc.getChild("writer_edit")
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"KEYCODE": "ESC"}))

        # Without the fix in place, this test would have crashed here
        self.xUITest.executeCommandWithParameters(".uno:BasicShapes.diamond", mkPropertyValues({"KeyModifier": 8192}))

        self.assertEqual(1, document.DrawPage.getCount())

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from uitest.framework import UITestCase
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict, type_text
import time
from uitest.debug import sleep
from uitest.uihelper.common import select_pos
#Bug 113252 - Basic Library Organizer is broken and closing dialogs crashes

class tdf113252(UITestCase):

   def test_tdf113252_macro_dialog(self):
        self.ui_test.create_doc_in_start_center("writer")
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()
        xWriterEdit = xWriterDoc.getChild("writer_edit")

        #Start LibreOffice. Go to Tools > Macros > Organize Macros > Basic
        self.ui_test.execute_dialog_through_command(".uno:MacroDialog")
        xDialog = self.xUITest.getTopFocusWindow()

        #Click Button Organizer
        xorganize = xDialog.getChild("organize")
        def handle_macro_dlg(dialog):

            xTabs = dialog.getChild("tabcontrol")
            select_pos(xTabs, "0")
            select_pos(xTabs, "1")
            select_pos(xTabs, "2")
            #Click on its button Close
            xCloseBtn = dialog.getChild("close")
            xCloseBtn.executeAction("CLICK", tuple())

        self.ui_test.execute_blocking_action(xorganize.executeAction, args=('CLICK', ()),
                dialog_handler=handle_macro_dlg)
        #Click button Close in the next dialog -> crash.

        xClose = xDialog.getChild("close")
        self.ui_test.close_dialog_through_button(xClose)

        self.assertEqual(get_state_as_dict(xWriterEdit)["CurrentPage"], "1")

        self.ui_test.close_doc()
# vim: set shiftwidth=4 softtabstop=4 expandtab:


# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict
import org.libreoffice.unotest
import pathlib

def get_url_for_data_file(file_name):
    return pathlib.Path(org.libreoffice.unotest.makeCopyFromTDOC(file_name)).as_uri()

class tdf131963(UITestCase):

    def test_tdf131963(self):
        writer_doc = self.ui_test.load_file(get_url_for_data_file("tdf131963.docx"))

        document = self.ui_test.get_component()
        self.assertEqual(11, document.CurrentController.PageCount)

        self.xUITest.executeCommand(".uno:SelectAll")
        self.xUITest.executeCommand(".uno:Copy")
        self.ui_test.execute_dialog_through_command(".uno:PasteSpecial")
        xDialog = self.xUITest.getTopFocusWindow()

        xList = xDialog.getChild('list')

        for childName in xList.getChildren():
            xChild = xList.getChild(childName)
            if get_state_as_dict(xChild)['Text'] == "Rich text formatting (RTF)":
                break

        xChild.executeAction("SELECT", tuple())
        self.assertEqual(get_state_as_dict(xList)['SelectEntryText'], "Rich text formatting (RTF)")

        xOkBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOkBtn)

        # Without the fix in place, it would have crashed here

        # tdf#133169: without the fix in place, it would have been 2 instead of 11
        self.assertEqual(11, document.CurrentController.PageCount)

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:




# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

from uitest.framework import UITestCase

class tdf131581(UITestCase):

    def test_tdf131581(self):
        self.ui_test.create_doc_in_start_center("writer")

        self.xUITest.getTopFocusWindow()

        self.ui_test.execute_dialog_through_command(".uno:OptionsTreeDialog")
        xDialogOpt = self.xUITest.getTopFocusWindow()

        xPages = xDialogOpt.getChild("pages")
        xLOEntry = xPages.getChild('0')
        xLOEntry.executeAction("EXPAND", tuple())
        xAdvancedEntry = xLOEntry.getChild('10')
        xAdvancedEntry.executeAction("SELECT", tuple())

        xExpertBtn = xDialogOpt.getChild("expertconfig")

        def handle_expert_dlg(dialog):
            # Without the fix in place, this would have hung
            xSearchBtn = dialog.getChild("searchButton")
            xSearchBtn.executeAction("CLICK", tuple())

            xOkBtn = dialog.getChild("ok")
            self.ui_test.close_dialog_through_button(xOkBtn)

        self.ui_test.execute_blocking_action(xExpertBtn.executeAction, args=('CLICK', ()),
                dialog_handler=handle_expert_dlg)

        xOKBtn = xDialogOpt.getChild("ok")
        xOKBtn.executeAction("CLICK", tuple())

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from uitest.uihelper.common import get_state_as_dict
from uitest.uihelper.common import select_pos
from uitest.uihelper.calc import enter_text_to_cell
from libreoffice.calc.document import get_cell_by_position
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict, type_text
from uitest.debug import sleep
import org.libreoffice.unotest
import pathlib

def get_url_for_data_file(file_name):
   return pathlib.Path(org.libreoffice.unotest.makeCopyFromTDOC(file_name)).as_uri()

#Bug 126017 - Crash swlo!SwNode::EndOfSectionIndex

class tdf126017(UITestCase):
   def test_tdf126017_crash_after_undo(self):
        writer_doc = self.ui_test.load_file(get_url_for_data_file("tdf126017.odt"))
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()

        #go to TOC
        self.ui_test.execute_modeless_dialog_through_command(".uno:SearchDialog")
        xDialog = self.xUITest.getTopFocusWindow()
        searchterm = xDialog.getChild("searchterm")
        searchterm.executeAction("TYPE", mkPropertyValues({"TEXT":"aasasas"}))
        xsearch = xDialog.getChild("search")
        xsearch.executeAction("CLICK", tuple())
        xcloseBtn = xDialog.getChild("close")
        self.ui_test.close_dialog_through_button(xcloseBtn)
        #edit index
        self.ui_test.execute_dialog_through_command(".uno:EditCurIndex")  #open index dialog
        xDiagIndex = self.xUITest.getTopFocusWindow()
        xOKBtn = xDiagIndex.getChild("ok")
        title = xDiagIndex.getChild("title")
        title.executeAction("TYPE", mkPropertyValues({"TEXT":"aaaa"}))
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.xUITest.executeCommand(".uno:Undo")

        self.ui_test.execute_dialog_through_command(".uno:EditCurIndex")  #open index dialog
        xDiagIndex = self.xUITest.getTopFocusWindow()
        xOKBtn = xDiagIndex.getChild("ok")
        title = xDiagIndex.getChild("title")
        title.executeAction("TYPE", mkPropertyValues({"TEXT":"aaaa"}))
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.xUITest.executeCommand(".uno:Undo")
        self.xUITest.executeCommand(".uno:Undo")

        self.assertEqual(document.Text.String.replace('\r\n', '\n')[1:7], "CRASHY")

        self.ui_test.close_doc()
# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from uitest.framework import UITestCase
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict
import time
from uitest.debug import sleep
from uitest.uihelper.common import select_pos

class autocorrectOptions(UITestCase):

   def test_autocorrect_options_writer(self):
        self.ui_test.create_doc_in_start_center("writer")
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()

        self.ui_test.execute_dialog_through_command(".uno:AutoCorrectDlg")
        xDialog = self.xUITest.getTopFocusWindow()
        xTabs = xDialog.getChild("tabcontrol")
        select_pos(xTabs, "0")       #tab replace
        origtext = xDialog.getChild("origtext")
        newtext = xDialog.getChild("newtext")
        xnew = xDialog.getChild("new")
        xdelete = xDialog.getChild("delete")
        xtabview = xDialog.getChild("tabview")
        xreset = xDialog.getChild("reset")
        nrRows = get_state_as_dict(xtabview)["VisibleCount"]

        self.assertTrue(int(nrRows) > 0)

        #add new rule
        origtext.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        origtext.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        origtext.executeAction("TYPE", mkPropertyValues({"TEXT":"::::"}))
        newtext.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        newtext.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        newtext.executeAction("TYPE", mkPropertyValues({"TEXT":"dvojtecky"}))
        xnew.executeAction("CLICK", tuple())
        nrRowsNew = get_state_as_dict(xtabview)["VisibleCount"]
        nrRowsDiff = int(nrRowsNew) - int(nrRows)
        self.assertEqual(nrRowsDiff, 1)  #we have +1 rule
        #delete rule
        origtext.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        origtext.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        origtext.executeAction("TYPE", mkPropertyValues({"TEXT":"::::"}))
        newtext.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        newtext.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        newtext.executeAction("TYPE", mkPropertyValues({"TEXT":"dvojtecky"}))
        xdelete.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(xtabview)["VisibleCount"], nrRows)   #we have default nr of rules

        select_pos(xTabs, "1")     #tab Exceptions
        #abbreviations
        abbrev = xDialog.getChild("abbrev")
        newabbrev = xDialog.getChild("newabbrev")
        delabbrev = xDialog.getChild("delabbrev")
        abbrevlist = xDialog.getChild("abbrevlist")

        nrRowsAbb = get_state_as_dict(abbrevlist)["VisibleCount"]

        self.assertTrue(int(nrRowsAbb) > 0)

        abbrev.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        abbrev.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        abbrev.executeAction("TYPE", mkPropertyValues({"TEXT":"qqqqq"}))
        newabbrev.executeAction("CLICK", tuple())
        nrRowsAbbNew = get_state_as_dict(abbrevlist)["VisibleCount"]
        nrRowsAbbDiff = int(nrRowsAbbNew) - int(nrRowsAbb)
        self.assertEqual(nrRowsAbbDiff, 1)  #we have +1 rule
        delabbrev.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(abbrevlist)["VisibleCount"], nrRowsAbb)   #we have default nr of rules

        #words with two initial capitals
        double = xDialog.getChild("double")
        newdouble = xDialog.getChild("newdouble")
        deldouble = xDialog.getChild("deldouble")
        doublelist = xDialog.getChild("doublelist")

        nrRowsDouble = get_state_as_dict(doublelist)["VisibleCount"]

        self.assertTrue(int(nrRowsDouble) > 0)

        double.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        double.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        double.executeAction("TYPE", mkPropertyValues({"TEXT":"QQqqq"}))
        newdouble.executeAction("CLICK", tuple())
        nrRowsDoubleNew = get_state_as_dict(doublelist)["VisibleCount"]
        nrRowsDoubleDiff = int(nrRowsDoubleNew) - int(nrRowsDouble) #convert string and
        self.assertEqual(nrRowsDoubleDiff, 1)  #we have +1 rule
        deldouble.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(doublelist)["VisibleCount"], nrRowsDouble)   #we have default nr of rules

        xCancelButton = xDialog.getChild("cancel")
        xCancelButton.executeAction("CLICK", tuple())

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from uitest.framework import UITestCase
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict, type_text
import time
from uitest.debug import sleep
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import select_pos

#Bug 122722 - Hiding characters will crash. Crash in: SwAttrIter::CtorInitAttrIter(SwTextNode &,SwScriptInfo &,SwTextFrame const *)

class tdf122722(UITestCase):
   def test_tdf122722_format_character_hidden(self):
        self.ui_test.create_doc_in_start_center("writer")
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()
        xWriterEdit = xWriterDoc.getChild("writer_edit")
        #1. Start LibreOffice
        #2. Create New Writer Document
        #3. Type "LibreOffice" in Writer
        type_text(xWriterEdit, "LibreOffice")
        #4. Select "LibreOffice" with mouse, and right click
        self.xUITest.executeCommand(".uno:SelectAll")
        self.assertEqual(document.Text.String[0:11], "LibreOffice")
        #5. Appear Context Menu, Character -> Character
        #6. Opened Character, Select "Font Effect" tab
        #7. Check Hidden, and click [OK]
        #8. Crash a LibreOffice
        self.ui_test.execute_dialog_through_command(".uno:FontDialog")
        xDialog = self.xUITest.getTopFocusWindow()
        xTabs = xDialog.getChild("tabcontrol")
        select_pos(xTabs, "1")

        xEffects = xDialog.getChild("effectslb")
        xRelief = xDialog.getChild("relieflb")
        xHidden = xDialog.getChild("hiddencb")
        xOverline = xDialog.getChild("overlinelb")
        xStrikeout = xDialog.getChild("strikeoutlb")
        xUnderline = xDialog.getChild("underlinelb")
        xEmphasis = xDialog.getChild("emphasislb")
        xPosition = xDialog.getChild("positionlb")

        xHidden.executeAction("CLICK", tuple())

        xOK = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOK)
        #un-hidden
        self.ui_test.execute_dialog_through_command(".uno:FontDialog")
        xDialog = self.xUITest.getTopFocusWindow()
        xTabs = xDialog.getChild("tabcontrol")
        select_pos(xTabs, "1")

        xEffects = xDialog.getChild("effectslb")
        xRelief = xDialog.getChild("relieflb")
        xHidden = xDialog.getChild("hiddencb")
        xOverline = xDialog.getChild("overlinelb")
        xStrikeout = xDialog.getChild("strikeoutlb")
        xUnderline = xDialog.getChild("underlinelb")
        xEmphasis = xDialog.getChild("emphasislb")
        xPosition = xDialog.getChild("positionlb")

        self.assertEqual(get_state_as_dict(xHidden)["Selected"], "true")
        xHidden.executeAction("CLICK", tuple())

        xOK = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOK)

        self.assertEqual(document.Text.String[0:11], "LibreOffice")

        self.ui_test.close_doc()
# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

from uitest.framework import UITestCase
from libreoffice.uno.propertyvalue import mkPropertyValues
import time
from uitest.uihelper.common import get_state_as_dict, type_text
from uitest.debug import sleep
from uitest.uihelper.common import select_pos

#specialcharacters.ui
class specialCharacter(UITestCase):

    def test_special_character(self):

        self.ui_test.create_doc_in_start_center("writer")

        xWriterDoc = self.xUITest.getTopFocusWindow()

        document = self.ui_test.get_component()

        self.ui_test.execute_dialog_through_command(".uno:InsertSymbol")  #specialCharacter dialog
        xDialog = self.xUITest.getTopFocusWindow()
        xCharSet = xDialog.getChild("showcharset")      #default charset

        xCharSet.executeAction("SELECT", mkPropertyValues({"COLUMN": "1", "ROW": "4"}))   #digit 4 selected

        xHexText = xDialog.getChild("hexvalue")
        xDecText = xDialog.getChild("decimalvalue")

        self.assertEqual(get_state_as_dict(xHexText)["Text"], "34")    # check the values Hex and decimal
        self.assertEqual(get_state_as_dict(xDecText)["Text"], "52")

        xCancelBtn = xDialog.getChild("cancel")
        self.ui_test.close_dialog_through_button(xCancelBtn)

        self.ui_test.execute_dialog_through_command(".uno:InsertSymbol")
        xDialog = self.xUITest.getTopFocusWindow()

        xComboFont = xDialog.getChild("fontlb")
        select_pos(xComboFont, "0")                        #select font
        xComboFont2 = xDialog.getChild("subsetlb")
        select_pos(xComboFont2, "0")                        #select font subset

        xSearchText = xDialog.getChild("search")                             #test search textBox
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"d"}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"i"}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"g"}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"i"}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"t"}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":" "}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"f"}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"o"}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"u"}))
        xSearchText.executeAction("TYPE", mkPropertyValues({"TEXT":"r"}))

# works locally and linux_gcc_release_64, but fails at linux_clang_dbgutil_64.
#Markus: Actually after a round of debugging I think the problem is actually that the test depends on the used font.
#Therefore, if the font is not available or not selected by default the test fails.
#        xCharSet = xDialog.getChild("searchcharset")    #another charset -> search charset
#        xCharSet.executeAction("SELECT", mkPropertyValues({"COLUMN": "0", "ROW": "0"}))   #digit 4 selected, we have only one result;
#        sleep(1)                                                  #try sleep here
#        xCharSet.executeAction("SELECT", mkPropertyValues({"COLUMN": "0", "ROW": "0"}))   #try it twice, because it works at local,but fail on gerrit
##gerrit:self.assertEqual(get_state_as_dict(xHexText)["Text"], "34")    # check the values for digit 4; AssertionError: '1' != '34'

#        xHexText = xDialog.getChild("hexvalue")
#        xDecText = xDialog.getChild("decimalvalue")
#        self.assertEqual(get_state_as_dict(xHexText)["Text"], "34")    # check the values for digit 4
#        self.assertEqual(get_state_as_dict(xDecText)["Text"], "52")

#        xAddFavBtn = xDialog.getChild("favbtn")
#        xAddFavBtn.executeAction("CLICK", tuple())     # Add to favorites button

#        xInsrBtn = xDialog.getChild("insert")
#        xInsrBtn.executeAction("CLICK", tuple())     # Insert to document

#        self.assertEqual(document.Text.String[0:1], "4")    # check inserted character

#        self.xUITest.executeCommand(".uno:Undo")
#        self.xUITest.executeCommand(".uno:Redo")   #undo, redo

#        self.assertEqual(document.Text.String[0:1], "4")    # check inserted character after undo, redo

        xCancelBtn = xDialog.getChild("cancel")
        self.ui_test.close_dialog_through_button(xCancelBtn)

        self.ui_test.close_doc()
# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from uitest.uihelper.common import get_state_as_dict
from uitest.uihelper.common import select_pos
from uitest.uihelper.calc import enter_text_to_cell
from libreoffice.calc.document import get_cell_by_position
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict, type_text
from uitest.debug import sleep
import org.libreoffice.unotest
import pathlib

def get_url_for_data_file(file_name):
    return pathlib.Path(org.libreoffice.unotest.makeCopyFromTDOC(file_name)).as_uri()

#Find similarities

class findSimilarities(UITestCase):
    def test_find_similarities(self):
        writer_doc = self.ui_test.load_file(get_url_for_data_file("findReplace.odt"))
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()
        xWriterEdit = xWriterDoc.getChild("writer_edit")
        #verify: we are on page 1
        self.assertEqual(get_state_as_dict(xWriterEdit)["CurrentPage"], "1")
        # search and replace dialog
        #find = seco
        self.ui_test.execute_modeless_dialog_through_command(".uno:SearchDialog")
        xDialog = self.xUITest.getTopFocusWindow()
        searchterm = xDialog.getChild("searchterm")
        searchterm.executeAction("TYPE", mkPropertyValues({"TEXT":"seco"}))  #seco
        # check similarities, button similarities, set values = 1; close dialog with OK
        similarity = xDialog.getChild("similarity")
        similaritybtn = xDialog.getChild("similaritybtn")
        similarity.executeAction("CLICK", tuple())
        def handle_similarity_dlg(dialog):
            otherfld = dialog.getChild("otherfld")
            longerfld = dialog.getChild("longerfld")
            shorterfld = dialog.getChild("shorterfld")
            otherfld.executeAction("DOWN", tuple())
            longerfld.executeAction("DOWN", tuple())
            shorterfld.executeAction("DOWN", tuple())
            xOKBtn = dialog.getChild("ok")
            self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.execute_blocking_action(similaritybtn.executeAction, args=('CLICK', ()),
                dialog_handler=handle_similarity_dlg)
        #open dialog again, verify values = 1; close dialog with cancel
        def handle_similarity_dlg2(dialog):
            otherfld = dialog.getChild("otherfld")
            longerfld = dialog.getChild("longerfld")
            shorterfld = dialog.getChild("shorterfld")
            self.assertEqual(get_state_as_dict(otherfld)["Text"], "1")
            self.assertEqual(get_state_as_dict(longerfld)["Text"], "1")
            self.assertEqual(get_state_as_dict(shorterfld)["Text"], "1")

            xCancelBtn = dialog.getChild("cancel")
            self.ui_test.close_dialog_through_button(xCancelBtn)

        self.ui_test.execute_blocking_action(similaritybtn.executeAction, args=('CLICK', ()),
                dialog_handler=handle_similarity_dlg2)

        xsearch = xDialog.getChild("search")
        xsearch.executeAction("CLICK", tuple())
        # Find next - not found
        self.assertEqual(get_state_as_dict(xWriterEdit)["CurrentPage"], "1")
        #find = secon
        searchterm.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        searchterm.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        searchterm.executeAction("TYPE", mkPropertyValues({"TEXT":"secon"}))
        # Find next - verify: we are on page 2
        xsearch = xDialog.getChild("search")
        xsearch.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(xWriterEdit)["CurrentPage"], "2")
        xWriterEdit.executeAction("TYPE", mkPropertyValues({"KEYCODE": "LEFT"})) #word "second" is selected
        #find = irst
        searchterm.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        searchterm.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        searchterm.executeAction("TYPE", mkPropertyValues({"TEXT":"irst"}))
        # Find next - verify: we are on page 1
        xsearch.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(xWriterEdit)["CurrentPage"], "1")

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



#! /usr/bin/env python
# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This file is part of the LibreOffice project.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

import os
import pathlib
import unittest
import unohelper
import uno

from org.libreoffice.unotest import UnoInProcess, makeCopyFromTDOC
from com.sun.star.beans import PropertyValue


class TestXStyleLoader(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls._uno = UnoInProcess()
        cls._uno.setUp()

    @classmethod
    def tearDownClass(cls):
        cls._uno.tearDown()

    def test_loadStyleFromStream(self):
        xDoc = self.__class__._uno.openEmptyWriterDoc()
        self.assertIsNotNone(xDoc)

        xServiceManager = self.__class__._uno.xContext.ServiceManager
        simpleFileAccess = xServiceManager.createInstance(
            "com.sun.star.ucb.SimpleFileAccess")
        xInputStream = simpleFileAccess.openFileRead(
            pathlib.Path(makeCopyFromTDOC("xstyleloader.odt")).as_uri())

        p1 = PropertyValue(Name="InputStream", Value=xInputStream)
        p2 = PropertyValue(Name="LoadTextStyles", Value=True)

        styles = xDoc.getStyleFamilies()
        styles.loadStylesFromURL("private:stream", [p1, p2])
        textStyles = styles.getByName("ParagraphStyles")
        self.assertTrue(textStyles.hasByName("Test_Template"))

        xDoc.close(True)


if __name__ == '__main__':
    unittest.main()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from uitest.uihelper.common import get_state_as_dict
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import change_measurement_unit
import org.libreoffice.unotest
import pathlib

def get_url_for_data_file(file_name):
    return pathlib.Path(org.libreoffice.unotest.makeCopyFromTDOC(file_name)).as_uri()

class tdf99711(UITestCase):
    def test_tdf99711(self):

        writer_doc = self.ui_test.load_file(get_url_for_data_file("shape.odt"))

        #set measurement to millimeters
        change_measurement_unit(self, "Millimeter")

        xWriterDoc = self.xUITest.getTopFocusWindow()
        xWriterEdit = xWriterDoc.getChild("writer_edit")

        self.xUITest.executeCommand(".uno:JumpToNextFrame")

        self.xUITest.executeCommand(".uno:Sidebar")
        xWriterEdit.executeAction("SIDEBAR", mkPropertyValues({"PANEL": "TextPropertyPanel"}))

        #wait until the sidebar is available
        self.ui_test.wait_until_child_is_available(xWriterEdit, 'selectwidth')
        self.assertEqual(get_state_as_dict(xWriterEdit.getChild('selectwidth'))['Text'], '10.00 mm')

        self.ui_test.wait_until_child_is_available(xWriterEdit, 'selectheight')
        self.assertEqual(get_state_as_dict(xWriterEdit.getChild('selectheight'))['Text'], '10.00 mm')

        self.xUITest.executeCommand(".uno:Sidebar")

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:




#!/usr/bin/env python
#
# This file is part of the LibreOffice project.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

import xml.sax
import sys


class ContentHandler(xml.sax.handler.ContentHandler):
    def __init__(self):
        self.tokens = []

    def startDocument(self):
        print("""
#include "ooxml/resourceids.hxx"
#include "ooxml/QNameToString.hxx"
#include "unordered_map"

namespace writerfilter
{

#ifdef DBG_UTIL

    /* ooxml */
    static const std::unordered_map<Id, const char*> g_QNameToStringMap {
""")

    def endDocument(self):
        print("""
   };

    std::string QNameToString(Id qName)
    {
        auto it = g_QNameToStringMap.find(qName);
        if (it == g_QNameToStringMap.end())
            return std::string();

        return it->second;
    }

#endif
}
""")

    def startElement(self, name, attrs):
        for k, v in list(attrs.items()):
            if k == "tokenid":
                if v.startswith("ooxml:"):
                    token = v.replace('ooxml:', '')
                    if token not in self.tokens:
                        print("""    { NS_ooxml::LN_%s, "ooxml:%s" },""" % (token, token))
                        self.tokens.append(token)


parser = xml.sax.make_parser()
parser.setContentHandler(ContentHandler())
parser.parse(sys.argv[1])

# vim:set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from uitest.uihelper.common import get_state_as_dict
from uitest.uihelper.common import select_pos
from uitest.uihelper.calc import enter_text_to_cell
from libreoffice.calc.document import get_cell_by_position
from libreoffice.uno.propertyvalue import mkPropertyValues
#uitest sw / Title Page dialog

class titlePage(UITestCase):
    def test_title_page(self):
        writer_doc = self.ui_test.create_doc_in_start_center("writer")
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()
        #dialog Title Page
        self.ui_test.execute_dialog_through_command(".uno:TitlePageDialog")
        xDialog = self.xUITest.getTopFocusWindow()
        #print(xDialog.getChildren())

        #select new Pages; nr of pages =2 (click UP), save; verify pageCount = 3
        newPages = xDialog.getChild("RB_INSERT_NEW_PAGES")
        newPages.executeAction("CLICK", tuple())
        xpageCount = xDialog.getChild("NF_PAGE_COUNT")
        xpageCount.executeAction("UP", tuple())
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)
        self.assertEqual(document.CurrentController.PageCount, 3)

        # check cancel button
        self.ui_test.execute_dialog_through_command(".uno:TitlePageDialog")
        xDialog = self.xUITest.getTopFocusWindow()
        xCancelBtn = xDialog.getChild("cancel")
        self.ui_test.close_dialog_through_button(xCancelBtn)
        self.assertEqual(document.CurrentController.PageCount, 3)

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

from uitest.framework import UITestCase
from uitest.uihelper.common import select_pos, get_state_as_dict
from libreoffice.uno.propertyvalue import mkPropertyValues

class tdf125104(UITestCase):

    def open_page_style_dialog(self):
        self.ui_test.execute_dialog_through_command(".uno:PageDialog")
        xDialog = self.xUITest.getTopFocusWindow()
        tabcontrol = xDialog.getChild("tabcontrol")
        select_pos(tabcontrol, "1")
        return xDialog.getChild("comboLayoutFormat")

    def set_combo_layout_format(self, dialog, format):
        comboLayoutFormat = dialog.getChild("comboLayoutFormat")
        props = {"TEXT": format}
        actionProps = mkPropertyValues(props)
        comboLayoutFormat.executeAction("SELECT", actionProps)
        okBtn = dialog.getChild("ok")
        self.ui_test.close_dialog_through_button(okBtn)

    def test_tdf125104_pageFormat_numbering(self):
        self.ui_test.create_doc_in_start_center("writer")
        document = self.ui_test.get_component()

        # insert page numbers on multiple pages
        self.xUITest.executeCommand(".uno:InsertPageNumberField")
        self.xUITest.executeCommand(".uno:InsertPagebreak")
        self.xUITest.executeCommand(".uno:InsertPageNumberField")
        text = document.Text.String.replace('\r\n', '\n')
        self.assertEqual(text[0:1], "1")
        self.assertEqual(text[2:3], "2")

        # Bug 125104 - Changing page numbering to "1st, 2nd, 3rd,..." causes crashes when trying to change Page settings later
        self.set_combo_layout_format(self.open_page_style_dialog(), "1st, 2nd, 3rd, ...")
        text = document.Text.String.replace('\r\n', '\n')
        self.assertEqual(text[0:3], "1st")
        self.assertEqual(text[4:7], "2nd")

        xDialog = self.open_page_style_dialog()
        comboLayoutFormat = xDialog.getChild("comboLayoutFormat")
        self.assertEqual(get_state_as_dict(comboLayoutFormat)["SelectEntryText"], "1st, 2nd, 3rd, ...")
        cancelBtn = xDialog.getChild("cancel")
        self.ui_test.close_dialog_through_button(cancelBtn)

        # change to devanagari alphabet format
        self.set_combo_layout_format(self.open_page_style_dialog(), "क, ख, ग, ...")
        text = document.Text.String.replace('\r\n', '\n')
        self.assertEqual(text[0:1], "क")
        self.assertEqual(text[2:3], "ख")

        # change to devanagari number format
        self.set_combo_layout_format(self.open_page_style_dialog(), "१, २, ३, ...")
        text = document.Text.String.replace('\r\n', '\n')
        self.assertEqual(text[0:1], "१")
        self.assertEqual(text[2:3], "२")

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from uitest.uihelper.common import get_state_as_dict
from uitest.uihelper.common import select_pos
from uitest.uihelper.calc import enter_text_to_cell
from libreoffice.calc.document import get_cell_by_position
from libreoffice.uno.propertyvalue import mkPropertyValues
#uitest sw / View-Zoom

class writerZoom(UITestCase):
    def test_zoom_writer(self):
        writer_doc = self.ui_test.create_doc_in_start_center("writer")
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()
        #dialog View-Zoom-Zoom
        self.ui_test.execute_dialog_through_command(".uno:Zoom")
        xDialog = self.xUITest.getTopFocusWindow()

        #select Optimal & Automatic - OK - open and verify
        optimal = xDialog.getChild("optimal")
        optimal.executeAction("CLICK", tuple())
        automatic = xDialog.getChild("automatic")
        automatic.executeAction("CLICK", tuple())
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.execute_dialog_through_command(".uno:Zoom")
        xDialog = self.xUITest.getTopFocusWindow()
        optimal = xDialog.getChild("optimal")
        automatic = xDialog.getChild("automatic")
        self.assertEqual(get_state_as_dict(optimal)["Checked"], "true")
        self.assertEqual(get_state_as_dict(automatic)["Checked"], "true")
        #select fit weight & Single page - OK - open and verify
        fitwandh = xDialog.getChild("fitwandh")
        singlepage = xDialog.getChild("singlepage")
        fitwandh.executeAction("CLICK", tuple())
        singlepage.executeAction("CLICK", tuple())
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.execute_dialog_through_command(".uno:Zoom")
        xDialog = self.xUITest.getTopFocusWindow()
        singlepage = xDialog.getChild("singlepage")
        fitwandh = xDialog.getChild("fitwandh")
        self.assertEqual(get_state_as_dict(singlepage)["Checked"], "true")
        self.assertEqual(get_state_as_dict(fitwandh)["Checked"], "true")
        #select fit width & columns  - OK - open and verify
        fitw = xDialog.getChild("fitw")
        columnssb = xDialog.getChild("columnssb")
        columns = xDialog.getChild("columns")
        fitw.executeAction("CLICK", tuple())
        columns.executeAction("CLICK", tuple())
        columnssb.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        columnssb.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        columnssb.executeAction("TYPE", mkPropertyValues({"TEXT":"3"}))
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.execute_dialog_through_command(".uno:Zoom")
        xDialog = self.xUITest.getTopFocusWindow()
        fitw = xDialog.getChild("fitw")
        columnssb = xDialog.getChild("columnssb")
        columns = xDialog.getChild("columns")
        self.assertEqual(get_state_as_dict(fitw)["Checked"], "true")
        self.assertEqual(get_state_as_dict(columns)["Checked"], "true")
        self.assertEqual(get_state_as_dict(columnssb)["Text"], "3")
        #select 100% & Automatic - OK - open and verify
        automatic = xDialog.getChild("automatic")
        x100pc = xDialog.getChild("100pc")
        x100pc.executeAction("CLICK", tuple())
        automatic.executeAction("CLICK", tuple())
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.execute_dialog_through_command(".uno:Zoom")
        xDialog = self.xUITest.getTopFocusWindow()
        automatic = xDialog.getChild("automatic")
        x100pc = xDialog.getChild("100pc")
        self.assertEqual(get_state_as_dict(automatic)["Checked"], "true")
        self.assertEqual(get_state_as_dict(x100pc)["Checked"], "true")
        #select variable 103% & Automatic - OK - open and verify
        variable = xDialog.getChild("variable")
        zoomsb = xDialog.getChild("zoomsb")
        variable.executeAction("CLICK", tuple())
        zoomsb.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        zoomsb.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        zoomsb.executeAction("TYPE", mkPropertyValues({"TEXT":"101"}))
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.execute_dialog_through_command(".uno:Zoom")
        xDialog = self.xUITest.getTopFocusWindow()
        variable = xDialog.getChild("variable")
        zoomsb = xDialog.getChild("zoomsb")
        self.assertEqual(get_state_as_dict(variable)["Checked"], "true")
        self.assertEqual(get_state_as_dict(zoomsb)["Text"], "101%")
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from uitest.framework import UITestCase
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict, type_text

import time
from uitest.debug import sleep
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import select_pos

class LibreLogoTest(UITestCase):
   LIBRELOGO_PATH = "vnd.sun.star.script:LibreLogo|LibreLogo.py$%s?language=Python&location=share"

   def createMasterScriptProviderFactory(self):
       xServiceManager = self.xContext.ServiceManager
       return xServiceManager.createInstanceWithContext(
           "com.sun.star.script.provider.MasterScriptProviderFactory",
           self.xContext)

   def getScript(self, command):
       xMasterScriptProviderFactory = self.createMasterScriptProviderFactory()
       document = self.ui_test.get_component()
       xScriptProvider = xMasterScriptProviderFactory.createScriptProvider(document)
       xScript = xScriptProvider.getScript(self.LIBRELOGO_PATH %command)
       self.assertIsNotNone(xScript, "xScript was not loaded")
       return xScript

   def logo(self, command):
        self.xUITest.executeCommand(self.LIBRELOGO_PATH %command)

   def test_librelogo(self):
        self.ui_test.create_doc_in_start_center("writer")
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()
        xWriterEdit = xWriterDoc.getChild("writer_edit")
        # to check the state of LibreLogo program execution
        xIsAlive = self.getScript("__is_alive__")

        # run a program with basic drawing commands FORWARD and RIGHT
        # using their abbreviated names FD and RT
        type_text(xWriterEdit, "fd 100 rt 45 fd 100")
        self.logo("run")
        # wait for LibreLogo program termination
        while xIsAlive.invoke((), (), ())[0]:
            pass
        # check shape count for
        # a) program running:
        # - turtle shape: result of program start
        # - line shape: result of turtle drawing
        # b) continuous line drawing (the regression
        # related to the fix of tdf#106792 resulted shorter line
        # segments than the turtle path and non-continuous line
        # drawing, ie. in this example, three line shapes
        # instead of a single one. See its fix in
        # commit 502e8785085f9e8b54ee383080442c2dcaf95b15)
        self.assertEqual(document.DrawPage.getCount(), 2)

        # check formatting by "magic wand"
        self.logo("__translate__")
        # a) check expansion of abbreviated commands : fd -> FORWARD, rt -> RIGHT,
        # b) check line breaking (fix for tdf#100941: new line instead of the text "\" and "n")
        self.assertEqual(document.Text.String.replace('\r\n', '\n'), "\nFORWARD 100 RIGHT 45 FORWARD 100")
        # c) check usage of real paragraphs instead of line break (tdf#120422)
        # first paragraph is empty (for working page break)
        self.assertEqual(document.Text.createEnumeration().nextElement().String, "")

        # function definitions and calls can be in arbitrary order
        document.Text.String = """
; dragon curve
TO x n
IF n = 0 [ STOP ]
x n-1
RIGHT 90
y n-1 ; it worked only as "y(n-1)"
FORWARD 10
END

TO y n
IF n = 0 [ STOP ]
FORWARD 10
x n-1
LEFT 90
y n-1
END

PICTURE ; start new line draw
x 3 ; draw only a few levels
"""
        self.logo("run")
        # wait for LibreLogo program termination
        while xIsAlive.invoke((), (), ())[0]:
            pass
        # new shape + previous two ones = 3
        self.assertEqual(document.DrawPage.getCount(), 3)

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from uitest.uihelper.common import get_state_as_dict
from uitest.uihelper.common import select_pos
from uitest.uihelper.calc import enter_text_to_cell
from libreoffice.calc.document import get_cell_by_position
from libreoffice.uno.propertyvalue import mkPropertyValues
from uitest.uihelper.common import get_state_as_dict, type_text
import org.libreoffice.unotest
import pathlib

def get_url_for_data_file(file_name):
    return pathlib.Path(org.libreoffice.unotest.makeCopyFromTDOC(file_name)).as_uri()

#Writer Table to text

class tableToText(UITestCase):
    def test_table_to_text(self):
        writer_doc = self.ui_test.load_file(get_url_for_data_file("tableToText.odt"))
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()
        #dialog Table to text - Tabs; verify
        self.ui_test.execute_dialog_through_command(".uno:ConvertTableToText")
        xDialog = self.xUITest.getTopFocusWindow()
        tabs = xDialog.getChild("tabs")
        tabs.executeAction("CLICK", tuple())
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)
        #verify
        self.assertEqual(document.Text.String[0:3], "a\ta")
        self.assertEqual(document.TextTables.getCount(), 0)
        #undo
        self.xUITest.executeCommand(".uno:Undo")
        self.assertEqual(document.TextTables.getCount(), 1)

        #dialog Table to text - Paragraph; verify
        self.ui_test.execute_dialog_through_command(".uno:ConvertTableToText")
        xDialog = self.xUITest.getTopFocusWindow()
        paragraph = xDialog.getChild("paragraph")
        paragraph.executeAction("CLICK", tuple())
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)
        #verify
        self.assertEqual(document.Text.String.replace('\r\n', '\n')[0:4], "a\na\n")
        self.assertEqual(document.TextTables.getCount(), 0)
        #undo
        self.xUITest.executeCommand(".uno:Undo")
        self.assertEqual(document.TextTables.getCount(), 1)

        #dialog Table to text - Semicolons; verify
        self.ui_test.execute_dialog_through_command(".uno:ConvertTableToText")
        xDialog = self.xUITest.getTopFocusWindow()
        semicolons = xDialog.getChild("semicolons")
        semicolons.executeAction("CLICK", tuple())
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)
        #verify
        self.assertEqual(document.Text.String.replace('\r\n', '\n')[0:6], "a;a\n;\n")
        self.assertEqual(document.TextTables.getCount(), 0)
        #undo
        self.xUITest.executeCommand(".uno:Undo")
        self.assertEqual(document.TextTables.getCount(), 1)

        #dialog Table to text - other; verify
        self.ui_test.execute_dialog_through_command(".uno:ConvertTableToText")
        xDialog = self.xUITest.getTopFocusWindow()
        other = xDialog.getChild("other")
        other.executeAction("CLICK", tuple())
        othered = xDialog.getChild("othered")
        othered.executeAction("TYPE", mkPropertyValues({"KEYCODE":"CTRL+A"}))
        othered.executeAction("TYPE", mkPropertyValues({"KEYCODE":"BACKSPACE"}))
        othered.executeAction("TYPE", mkPropertyValues({"TEXT":":"}))
        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)
        #verify
        self.assertEqual(document.Text.String.replace('\r\n', '\n')[0:6], "a:a\n:\n")
        self.assertEqual(document.TextTables.getCount(), 0)
        #undo
        self.xUITest.executeCommand(".uno:Undo")
        self.assertEqual(document.TextTables.getCount(), 1)

        self.ui_test.close_doc()
# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
from uitest.framework import UITestCase
from uitest.uihelper.common import get_state_as_dict
from libreoffice.uno.propertyvalue import mkPropertyValues
import org.libreoffice.unotest
import pathlib
import time

def get_url_for_data_file(file_name):
    return pathlib.Path(org.libreoffice.unotest.makeCopyFromTDOC(file_name)).as_uri()

class dropDownFormFieldDialog(UITestCase):

    def test_add_new_items(self):

        # open a file with an empty form field
        writer_doc = self.ui_test.load_file(get_url_for_data_file("empty_drop_down_form_field.odt"))
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()

        # open the dialog (cursor is at the field)
        self.ui_test.execute_dialog_through_command(".uno:ControlProperties")
        xDialog = self.xUITest.getTopFocusWindow()

        itemEntry = xDialog.getChild("item_entry")
        addButton = xDialog.getChild("add_button")
        itemsList = xDialog.getChild("items_treeview")

        # initial state
        self.assertEqual(get_state_as_dict(itemEntry)["Text"], "")
        self.assertEqual(get_state_as_dict(addButton)["Enabled"], "false")
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "0")

        # add some new items
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"1000"}))
        self.assertEqual(get_state_as_dict(addButton)["Enabled"], "true")
        addButton.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(addButton)["Enabled"], "false")
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"2000"}))
        addButton.executeAction("CLICK", tuple())
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"3000"}))
        addButton.executeAction("CLICK", tuple())
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"4000"}))
        addButton.executeAction("CLICK", tuple())

        # check whether the items are there in the list
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "4")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "2000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("3"))["Text"], "4000")

        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        # check whether items are the same after reopening
        self.ui_test.execute_dialog_through_command(".uno:ControlProperties")
        xDialog = self.xUITest.getTopFocusWindow()

        itemsList = xDialog.getChild("items_treeview")
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "4")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "2000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("3"))["Text"], "4000")

        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.close_doc()

    def test_remove_items(self):

        # open a file with an empty form field
        writer_doc = self.ui_test.load_file(get_url_for_data_file("empty_drop_down_form_field.odt"))
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()

        # open the dialog (cursor is at the field)
        self.ui_test.execute_dialog_through_command(".uno:ControlProperties")
        xDialog = self.xUITest.getTopFocusWindow()

        itemEntry = xDialog.getChild("item_entry")
        addButton = xDialog.getChild("add_button")
        itemsList = xDialog.getChild("items_treeview")
        removeButton = xDialog.getChild("remove_button")

        # initial state
        self.assertEqual(get_state_as_dict(itemEntry)["Text"], "")
        self.assertEqual(get_state_as_dict(addButton)["Enabled"], "false")
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "0")
        self.assertEqual(get_state_as_dict(removeButton)["Enabled"], "false")

        # add some new items
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"1000"}))
        self.assertEqual(get_state_as_dict(addButton)["Enabled"], "true")
        addButton.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(removeButton)["Enabled"], "true")
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"2000"}))
        addButton.executeAction("CLICK", tuple())
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"3000"}))
        addButton.executeAction("CLICK", tuple())
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"4000"}))
        addButton.executeAction("CLICK", tuple())

        # check whether the items are there in the list
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "4")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "2000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("3"))["Text"], "4000")

        # select an item from the list and remove it
        itemsList.getChild("1").executeAction("SELECT", tuple());
        removeButton.executeAction("CLICK", tuple())

        # check whether the right item was removed
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "3")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "4000")

        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        # check whether items are the same after reopening
        self.ui_test.execute_dialog_through_command(".uno:ControlProperties")
        xDialog = self.xUITest.getTopFocusWindow()

        itemsList = xDialog.getChild("items_treeview")
        removeButton = xDialog.getChild("remove_button")
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "3")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "4000")

        # remove all items
        itemsList.getChild("1").executeAction("SELECT", tuple());
        removeButton.executeAction("CLICK", tuple())
        removeButton.executeAction("CLICK", tuple())
        removeButton.executeAction("CLICK", tuple())

        self.assertEqual(get_state_as_dict(removeButton)["Enabled"], "false")
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "0")

        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.close_doc()

    def test_move_items(self):

        # open a file with an empty form field
        writer_doc = self.ui_test.load_file(get_url_for_data_file("empty_drop_down_form_field.odt"))
        document = self.ui_test.get_component()
        xWriterDoc = self.xUITest.getTopFocusWindow()

        # open the dialog (cursor is at the field)
        self.ui_test.execute_dialog_through_command(".uno:ControlProperties")
        xDialog = self.xUITest.getTopFocusWindow()

        itemEntry = xDialog.getChild("item_entry")
        addButton = xDialog.getChild("add_button")
        itemsList = xDialog.getChild("items_treeview")
        upButton = xDialog.getChild("up_button")
        downButton = xDialog.getChild("down_button")

        # initial state
        self.assertEqual(get_state_as_dict(itemEntry)["Text"], "")
        self.assertEqual(get_state_as_dict(addButton)["Enabled"], "false")
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "0")
        self.assertEqual(get_state_as_dict(upButton)["Enabled"], "false")
        self.assertEqual(get_state_as_dict(downButton)["Enabled"], "false")

        # add some new items
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"1000"}))
        self.assertEqual(get_state_as_dict(addButton)["Enabled"], "true")
        addButton.executeAction("CLICK", tuple())
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"2000"}))
        addButton.executeAction("CLICK", tuple())
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"3000"}))
        addButton.executeAction("CLICK", tuple())
        itemEntry.executeAction("TYPE", mkPropertyValues({"TEXT":"4000"}))
        addButton.executeAction("CLICK", tuple())

        # check whether the items are there in the list
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "4")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "2000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("3"))["Text"], "4000")

        # select an item from the list and move it up
        itemsList.getChild("1").executeAction("SELECT", tuple())
        self.assertEqual(get_state_as_dict(upButton)["Enabled"], "true")
        self.assertEqual(get_state_as_dict(downButton)["Enabled"], "true")
        upButton.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(upButton)["Enabled"], "false")
        self.assertEqual(get_state_as_dict(downButton)["Enabled"], "true")

        # check whether the item was correctly moved
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "4")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "2000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("3"))["Text"], "4000")

        # move down the selected item
        downButton.executeAction("CLICK", tuple())
        downButton.executeAction("CLICK", tuple())
        downButton.executeAction("CLICK", tuple())
        self.assertEqual(get_state_as_dict(upButton)["Enabled"], "true")
        self.assertEqual(get_state_as_dict(downButton)["Enabled"], "false")

        # check whether the item was correctly moved
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "4")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "4000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("3"))["Text"], "2000")

        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        # check whether items are the same after reopening
        self.ui_test.execute_dialog_through_command(".uno:ControlProperties")
        xDialog = self.xUITest.getTopFocusWindow()

        itemsList = xDialog.getChild("items_treeview")
        self.assertEqual(get_state_as_dict(itemsList)["Children"], "4")
        self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "3000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "4000")
        self.assertEqual(get_state_as_dict(itemsList.getChild("3"))["Text"], "2000")

        xOKBtn = xDialog.getChild("ok")
        self.ui_test.close_dialog_through_button(xOKBtn)

        self.ui_test.close_doc()

    def test_drop_down_after_import(self):

        files = ["drop_down_form_field.odt", "drop_down_form_field.doc", "drop_down_form_field.docx"]
        for file in files:
            # open a file with a drop-down for field with items and selection
            writer_doc = self.ui_test.load_file(get_url_for_data_file(file))
            document = self.ui_test.get_component()
            xWriterDoc = self.xUITest.getTopFocusWindow()

            # open the dialog (cursor is at the field)
            self.ui_test.execute_dialog_through_command(".uno:ControlProperties")
            xDialog = self.xUITest.getTopFocusWindow()

            itemsList = xDialog.getChild("items_treeview")

            # check whether the items are there in the list
            self.assertEqual(get_state_as_dict(itemsList)["Children"], "4")
            self.assertEqual(get_state_as_dict(itemsList.getChild("0"))["Text"], "1000")
            self.assertEqual(get_state_as_dict(itemsList.getChild("1"))["Text"], "2000")
            self.assertEqual(get_state_as_dict(itemsList.getChild("2"))["Text"], "3000")
            self.assertEqual(get_state_as_dict(itemsList.getChild("3"))["Text"], "4000")

            self.assertEqual(get_state_as_dict(itemsList)["SelectEntryText"], "3000")

            xOKBtn = xDialog.getChild("ok")
            self.ui_test.close_dialog_through_button(xOKBtn)

            self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



#!/usr/bin/env python3
# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This file is part of the LibreOffice project.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#

import os
import sys
import argparse

def parse_line(line):
    """
    This function parses a line from log file
    and returns the parsed values as a python dictionary
    """
    if (line == ""):
        return
    dict = {}
    if "{" in line:
        start_index_of_parameters = line.find("{")
        end_index_of_parameters = line.find("}") + 1
        parameters = line[start_index_of_parameters:end_index_of_parameters]
        if parameters != "":
            dict["parameters"] = parameters
        line = line[:start_index_of_parameters-1]
    word_list = line.split()
    dict["keyword"] = word_list[0]

    for index in range(1,len(word_list)):
        key, val = word_list[index].split(":",1)
        dict[key] = val
    return dict

def parse_args():
    """
    This function parses the command-line arguments
    to get the input and output file details
    """
    parser = argparse.ArgumentParser(description = "Generate a UI test file from log")
    parser.add_argument("input_address", type = str, help = "The log file address")
    parser.add_argument("output_address", type = str, help = "The test file address")
    parser.add_argument("-d", "--document", metavar = "", help = "Address of the document to be opened")
    args = parser.parse_args()
    return args

def get_log_file(input_address):
    try:
        with open(input_address) as f:
            content = f.readlines()
    except IOError as err:
        print("IO error: {0}".format(err))
        print("Use " + os.path.basename(sys.argv[0]) + " -h to get usage instructions")
        sys.exit(1)

    content = [x.strip() for x in content if not x.startswith("Action on element")]
    return content

def initiate_test_generation(address):
    try:
        f = open(address,"w")
    except IOError as err:
        print("IO error: {0}".format(err))
        print("Use " + os.path.basename(sys.argv[0]) + " -h to get usage instructions")
        sys.exit(1)
    initial_text = \
    "# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-\n\n" + \
    "from uitest.framework import UITestCase\n" + \
    "from libreoffice.uno.propertyvalue import mkPropertyValues\n" + \
    "import importlib\n\n" + \
    "class TestClass(UITestCase):\n" + \
    "    def test_function(self):\n"
    f.write(initial_text)
    return f

def get_coupling_type(line1, line2):
    """
    This function checks if two consecutive lines of log file
    refer to the same event
    """
    action_dict1 = parse_line(line1)
    action_dict2 = parse_line(line2)

    if action_dict1["keyword"] == "CommandSent" and \
        action_dict2["keyword"] == "ModalDialogExecuted":
        return "COMMAND_MODAL_COUPLE"

    elif action_dict1["keyword"] == "CommandSent" and \
        action_dict2["keyword"] == "ModelessDialogConstructed":
        return "COMMAND_MODELESS_COUPLE"

    elif action_dict1["keyword"] == "ButtonUIObject" and \
        action_dict2["keyword"] == "DialogClosed":
        return "BUTTON_DIALOGCLOSE_COUPLE"

    elif "parameters" in action_dict1 and \
        "KEYCODE" in action_dict1["parameters"] and \
        action_dict2["keyword"] == "CommandSent":
        return "REDUNDANT_COUPLE"

    return "NOT_A_COUPLE"

def check_app_starting_action(action_dict):
    app_starter_button_ids = \
    set(["draw_all", "impress_all", "calc_all" , "writer_all", "database_all", "math_all"])

    if action_dict["keyword"] == "ButtonUIObject" and action_dict["Action"] == "CLICK" and \
    action_dict["Id"] in app_starter_button_ids:
        return True
    return False

def get_test_line_from_one_log_line(log_line):
    action_dict = parse_line(log_line)
    test_line = "        "
    if action_dict["keyword"].endswith("UIObject"):
        parent = action_dict["Parent"]
        if (check_app_starting_action(action_dict)):
            test_line +=\
            "MainDoc = self.ui_test.create_doc_in_start_center(\"" + \
            action_dict["Id"][:-4] +"\")\n        MainWindow = " + \
            "self.xUITest.getTopFocusWindow()\n"
            return test_line
        else:
            if (parent == ""):
                parent = "MainWindow"
            test_line += \
            action_dict["Id"] + " = " + parent + ".getChild(\"" + \
            action_dict["Id"] + "\")\n        " + \
            action_dict["Id"] + ".executeAction(\"" + \
            action_dict["Action"] + "\""
            if "parameters" in action_dict:
                test_line +=  ", mkPropertyValues(" + \
                action_dict["parameters"] + "))\n"
            else:
                test_line += ",tuple())\n"
            return test_line
    elif action_dict["keyword"] == "CommandSent":
        if "parameters" not in action_dict:
            test_line += "self.xUITest.executeCommand(\"" + \
            action_dict["Name"] + "\")\n"
            return test_line
        else:
            test_line += "self.xUITest.executeCommandWithParameters(\"" + \
            action_dict["Name"] + "\", mkPropertyValues(" + action_dict["parameters"] + \
            "))\n"
            return test_line
    elif action_dict["keyword"] == "ModalDialogExecuted" or \
        action_dict["keyword"] == "ModelessDialogConstructed":
        test_line += action_dict["Id"] + " = " + "self.xUITest.getTopFocusWindow()\n"
        return test_line

    return ""

def get_test_line_from_two_log_lines(log_line1,log_line2):
    coupling_type = get_coupling_type(log_line1, log_line2)
    action_dict1 = parse_line(log_line1)
    action_dict2 = parse_line(log_line2)
    test_line = "        "
    if coupling_type == "COMMAND_MODAL_COUPLE":
        test_line += \
        "self.ui_test.execute_dialog_through_command(\"" + \
        action_dict1["Name"] + "\")\n        " + \
        action_dict2["Id"] + " = self.xUITest.getTopFocusWindow()\n"
    elif coupling_type == "COMMAND_MODELESS_COUPLE":
        test_line += \
        "self.ui_test.execute_modeless_dialog_through_command(\"" + \
        action_dict1["Name"] + "\")\n        " + \
        action_dict2["Id"] + " = self.xUITest.getTopFocusWindow()\n"
    elif coupling_type == "BUTTON_DIALOGCLOSE_COUPLE":
        test_line += \
        action_dict1["Id"] + " = " + action_dict1["Parent"] + ".getChild(\"" + \
        action_dict1["Id"] + "\")\n        self.ui_test.close_dialog_through_button(" + \
        action_dict1["Id"] + ")\n"
    return test_line

def main():
    args = parse_args()
    log_lines = get_log_file(args.input_address)
    output_stream = initiate_test_generation(args.output_address)
    if args.document is not None:
        output_line = "        pathmodule = importlib.import_module(\"uitest.path\")\n" + \
        "        doc_path = pathmodule.get_srcdir_url() + \"" + args.document + "\"\n" + \
        "        MainDoc = self.ui_test.load_file(doc_path)\n" + \
        "        MainWindow = self.xUITest.getTopFocusWindow()\n"
        output_stream.write(output_line)
    line_number = 0
    while line_number < len(log_lines):
        if line_number == len(log_lines)-1 or \
            get_coupling_type(log_lines[line_number],log_lines[line_number + 1]) == "NOT_A_COUPLE":
            test_line = get_test_line_from_one_log_line(log_lines[line_number])
            output_stream.write(test_line)
            line_number += 1
        elif get_coupling_type(log_lines[line_number],log_lines[line_number + 1]) == "REDUNDANT_COUPLE":
            line_number += 1
        else:
            test_line = get_test_line_from_two_log_lines(log_lines[line_number],log_lines[line_number + 1])
            output_stream.write(test_line)
            line_number += 2
    output_stream.write("        self.ui_test.close_doc()")
    output_stream.write("\n\n# vim: set shiftwidth=4 softtabstop=4 expandtab:")
    output_stream.close()

if __name__ == '__main__':
    main()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-

from uitest.framework import UITestCase
from uitest.uihelper.common import select_pos
from uitest.uihelper.common import get_state_as_dict
import org.libreoffice.unotest
import pathlib

def get_url_for_data_file(file_name):
    return pathlib.Path(org.libreoffice.unotest.makeCopyFromTDOC(file_name)).as_uri()

class tdf135693(UITestCase):

    def test_tdf135693(self):
        writer_doc = self.ui_test.load_file(get_url_for_data_file("tdf135693.odt"))

        self.xUITest.getTopFocusWindow()

        self.xUITest.executeCommand(".uno:GoRight")
        self.xUITest.executeCommand(".uno:GoDown")

        # Without the fix in place, this would have crashed here
        self.ui_test.execute_dialog_through_command(".uno:TableDialog")

        xTableDlg = self.xUITest.getTopFocusWindow()

        xTabs = xTableDlg.getChild("tabcontrol")
        select_pos(xTabs, "0")

        # Check we are in the right table
        self.assertEqual("Table1", get_state_as_dict(xTabs.getChild('name'))['Text'])

        xok = xTableDlg.getChild("ok")
        self.ui_test.close_dialog_through_button(xok)

        self.ui_test.close_doc()

# vim: set shiftwidth=4 softtabstop=4 expandtab:



#! /usr/bin/env python
# -*- tab-width: 4; indent-tabs-mode: nil; py-indent-offset: 4 -*-
#
# This file is part of the LibreOffice project.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
import unittest
from org.libreoffice.unotest import UnoInProcess


class XModifiable2(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls._uno = UnoInProcess()
        cls._uno.setUp()

    @classmethod
    def tearDownClass(cls):
        cls._uno.tearDown()

    def test_XModifiable2(self):
        # initialization
        xDoc = self._uno.openEmptyWriterDoc()

        # perform unit test
        self.assertTrue(xDoc.isSetModifiedEnabled())

        self.assertTrue(xDoc.disableSetModified())
        self.assertFalse(xDoc.isSetModifiedEnabled())

        self.assertFalse(xDoc.enableSetModified())
        self.assertTrue(xDoc.isSetModifiedEnabled())

        # clean up
        xDoc.close(True)

    def test_XModifiable2_Double(self):
        # initialization
        xDoc = self._uno.openEmptyWriterDoc()

        # perform unit test
        self.assertTrue(xDoc.isSetModifiedEnabled())

        # try to disable
        # Expected return value:
        #   `TRUE` the changing of the modified state was disabled
        self.assertTrue(xDoc.disableSetModified())
        self.assertFalse(xDoc.isSetModifiedEnabled())

        # try to disable twice
        # Expected return value:
        #   `FALSE` the changing of the modified state was already disabled
        self.assertFalse(xDoc.disableSetModified())
        self.assertFalse(xDoc.isSetModifiedEnabled())

        # try to disable third time
        # Expected return value:
        #   `FALSE` the changing of the modified state was already disabled
        #   i.e. the same as in previous call
        self.assertFalse(xDoc.disableSetModified())
        self.assertFalse(xDoc.isSetModifiedEnabled())

        # try to enable
        # Expected return value:
        #   `FALSE` the changing of the modified state was enabled
        self.assertFalse(xDoc.enableSetModified())
        self.assertTrue(xDoc.isSetModifiedEnabled())

        # try to enable twice
        # Expected return value:
        #   `TRUE` the changing of the modified state was already enabled
        self.assertTrue(xDoc.enableSetModified())
        self.assertTrue(xDoc.isSetModifiedEnabled())

        # try to enable third time
        # Expected return value:
        #   `TRUE` the changing of the modified state was already enabled
        #   i.e. the same as in previous call
        self.assertTrue(xDoc.enableSetModified())
        self.assertTrue(xDoc.isSetModifiedEnabled())

        # clean up
        xDoc.close(True)

    def test_XModifiable2_setModified(self):
        # initialization
        xDoc = self._uno.openEmptyWriterDoc()

        # perform unit test
        # try to set modified flag when modification enabled
        self.assertTrue(xDoc.isSetModifiedEnabled())

        self.assertFalse(xDoc.isModified())
        xDoc.setModified(True)
        self.assertTrue(xDoc.isModified())

        xDoc.setModified(False)
        self.assertFalse(xDoc.isModified())

        # try to set modified flag when modification disabled
        self.assertTrue(xDoc.disableSetModified())
        self.assertFalse(xDoc.isSetModifiedEnabled())

        self.assertFalse(xDoc.isModified())
        xDoc.setModified(True)
        self.assertFalse(xDoc.isModified())

        self.assertFalse(xDoc.enableSetModified())
        self.assertTrue(xDoc.isSetModifiedEnabled())

        # document is still not modified
        self.assertFalse(xDoc.isModified())

        # try to set modified flag when modification enabled
        # and when we have changed the modification possibility
        self.assertTrue(xDoc.isSetModifiedEnabled())

        self.assertFalse(xDoc.isModified())
        xDoc.setModified(True)
        self.assertTrue(xDoc.isModified())

        xDoc.setModified(False)
        self.assertFalse(xDoc.isModified())

        # clean up
        xDoc.close(True)

    def test_setModified_ByContent(self):
        # initialization
        xDoc = self._uno.openEmptyWriterDoc()

        # perform unit test:
        #        set modified flag using text editing
        #        when modification of the flag is enabled
        self.assertTrue(xDoc.isSetModifiedEnabled())
        self.assertFalse(xDoc.isModified())

        cursor = xDoc.Text.createTextCursor()
        xDoc.Text.insertString(cursor, "The first paragraph", 0)

        self.assertTrue(xDoc.isSetModifiedEnabled())
        self.assertTrue(xDoc.isModified())

        # clean up
        xDoc.close(True)

    def test_setModified_ByContent_Blocked(self):
        # initialization
        xDoc = self._uno.openEmptyWriterDoc()

        # perform unit test:
        #       it is unable to set modified flag using text editing
        #       when modification of the flag was disabled
        self.assertTrue(xDoc.disableSetModified())
        self.assertFalse(xDoc.isSetModifiedEnabled())
        self.assertFalse(xDoc.isModified())

        cursor = xDoc.Text.createTextCursor()
        xDoc.Text.insertString(cursor, "The first paragraph", 0)

        self.assertFalse(xDoc.isSetModifiedEnabled())
        self.assertFalse(xDoc.isModified())

        # clean up
        xDoc.close(True)

    def test_WriteProtectedDocument(self):
        # initialization
        xDoc = self._uno.openTemplateFromTDOC('WriteProtected.odt')

        # perform unit test:
        #       it is unable to set modified flag using text editing
        #       when modification of the flag was disabled as
        #       ODT file was marked to be opened as read-only
        self.assertFalse(xDoc.isSetModifiedEnabled())
        self.assertFalse(xDoc.isModified())

        cursor = xDoc.Text.createTextCursor()
        xDoc.Text.insertString(cursor, "The first paragraph", 0)

        self.assertFalse(xDoc.isSetModifiedEnabled())
        self.assertFalse(xDoc.isModified())

        # clean up
        xDoc.close(True)


if __name__ == '__main__':
    unittest.main()

# vim: set shiftwidth=4 softtabstop=4 expandtab:
